@@ -118,13 +118,13 @@ wiener_map( unsigned int n )
 }
 
 static void
-test_keys( ELG_secret_key *sk, unsigned nbits )
+test_keys( ELG_secret_key *sk, unsigned int nbits )
 {
     ELG_public_key pk;
     MPI test = mpi_alloc( 0 );
-    MPI out1_a = mpi_alloc( nbits / BITS_PER_MPI_LIMB );
-    MPI out1_b = mpi_alloc( nbits / BITS_PER_MPI_LIMB );
-    MPI out2 = mpi_alloc( nbits / BITS_PER_MPI_LIMB );
+    MPI out1_a = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
+    MPI out1_b = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
+    MPI out2   = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
 
     pk.p = sk->p;
     pk.g = sk->g;
@@ -133,7 +133,7 @@ test_keys( ELG_secret_key *sk, unsigned nbits )
     /*mpi_set_bytes( test, nbits, get_random_byte, 0 );*/
     {	char *p = get_random_bits( nbits, 0, 0 );
 	mpi_set_buffer( test, p, (nbits+7)/8, 0 );
-	m_free(p);
+	xfree(p);
     }
 
     do_encrypt( out1_a, out1_b, test, &pk );
@@ -183,7 +183,7 @@ gen_k( MPI p, int small_k )
     mpi_sub_ui( p_1, p, 1);
     for(;;) {
 	if( !rndbuf || nbits < 32 ) {
-	    m_free(rndbuf);
+	    xfree(rndbuf);
 	    rndbuf = get_random_bits( nbits, 1, 1 );
 	}
 	else { /* Change only some of the higher bits. */
@@ -194,7 +194,7 @@ gen_k( MPI p, int small_k )
 	     */
 	    char *pp = get_random_bits( 32, 1, 1 );
 	    memcpy( rndbuf,pp, 4 );
-	    m_free(pp);
+	    xfree(pp);
 	}
 	mpi_set_buffer( k, rndbuf, nbytes, 0 );
 
@@ -217,7 +217,7 @@ gen_k( MPI p, int small_k )
 	}
     }
   found:
-    m_free(rndbuf);
+    xfree(rndbuf);
     if( DBG_CIPHER )
 	progress('\n');
     mpi_free(p_1);
@@ -244,9 +244,9 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
     unsigned int xbits;
     byte *rndbuf;
 
-    p_min1 = mpi_alloc( (nbits+BITS_PER_MPI_LIMB-1)/BITS_PER_MPI_LIMB );
-    temp   = mpi_alloc( (nbits+BITS_PER_MPI_LIMB-1)/BITS_PER_MPI_LIMB );
-    qbits = wiener_map( nbits );
+    p_min1 = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
+    temp   = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
+    qbits  = wiener_map ( nbits );
     if( qbits & 1 ) /* better have a even one */
 	qbits++;
     g = mpi_alloc(1);
@@ -271,7 +271,7 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
     xbits = qbits * 3 / 2;
     if( xbits >= nbits )
 	BUG();
-    x = mpi_alloc_secure( xbits/BITS_PER_MPI_LIMB );
+    x = mpi_alloc_secure ( mpi_nlimb_hint_from_nbits (xbits) );
     if( DBG_CIPHER )
 	log_debug("choosing a random x of size %u", xbits );
     rndbuf = NULL;
@@ -280,13 +280,13 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
 	    progress('.');
 	if( rndbuf ) { /* change only some of the higher bits */
 	    if( xbits < 16 ) {/* should never happen ... */
-		m_free(rndbuf);
+		xfree(rndbuf);
 		rndbuf = get_random_bits( xbits, 2, 1 );
 	    }
 	    else {
 		char *r = get_random_bits( 16, 2, 1 );
 		memcpy(rndbuf, r, 16/8 );
-		m_free(r);
+		xfree(r);
 	    }
 	}
 	else
@@ -294,9 +294,9 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
 	mpi_set_buffer( x, rndbuf, (xbits+7)/8, 0 );
 	mpi_clear_highbit( x, xbits+1 );
     } while( !( mpi_cmp_ui( x, 0 )>0 && mpi_cmp( x, p_min1 )<0 ) );
-    m_free(rndbuf);
+    xfree(rndbuf);
 
-    y = mpi_alloc(nbits/BITS_PER_MPI_LIMB);
+    y = mpi_alloc ( mpi_nlimb_hint_from_nbits (nbits) );
     mpi_powm( y, g, x, p );
 
     if( DBG_CIPHER ) {
