@@ -25,11 +25,12 @@
 
 /*-- random.c --*/
 void random_dump_stats(void);
-void secure_random_alloc(void);
+void secure_randoxmalloc(void);
 void set_random_seed_file(const char *);
 void update_random_seed_file(void);
 int  quick_random_gen( int onoff );
 int  random_is_faked(void);
+void random_disable_locking (void);
 void randomize_buffer( byte *buffer, size_t length, int level );
 byte *get_random_bits( size_t nbits, int level, int secure );
 void fast_random_poll( void );
