@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
- * $Id: app-openpgp.c 3982 2006-01-01 17:48:54Z dshaw $
+ * $Id: app-openpgp.c 4178 2006-06-28 09:37:42Z wk $
  */
 
 #include <config.h>
@@ -1789,7 +1789,7 @@ do_writekey (app_t app, ctrl_t ctrl,
   if (rsa_e_len < 4)
     {
       /* Right justify E. */
-      memmove (tp+4-rsa_e_len, tp, 4-rsa_e_len);
+      memmove (tp+4-rsa_e_len, tp, rsa_e_len);
       memset (tp, 0, 4-rsa_e_len);
     }                 
   tp += 4;
