@@ -1,5 +1,5 @@
 /* exec.c - generic call-a-program code
- * Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -51,7 +52,7 @@ int exec_write(struct exec_info **info,const char *program,
 
 int exec_read(struct exec_info *info) { return G10ERR_GENERAL; }
 int exec_finish(struct exec_info *info) { return G10ERR_GENERAL; }
-int set_exec_path(const char *path,int method) { return G10ERR_GENERAL; }
+int set_exec_path(const char *path) { return G10ERR_GENERAL; }
 
 #else /* ! NO_EXEC */
 
@@ -71,7 +72,7 @@ static int win_system(const char *command)
 
   /* We must use a copy of the command as CreateProcess modifies this
      argument. */
-  string=m_strdup(command);
+  string=xstrdup(command);
 
   memset(&pi,0,sizeof(pi));
   memset(&si,0,sizeof(si));
@@ -85,35 +86,23 @@ static int win_system(const char *command)
 
   CloseHandle(pi.hProcess);
   CloseHandle(pi.hThread);
-  m_free(string);
+  xfree(string);
 
   return 0;
 }
 #endif
 
-/* method==0 to replace current $PATH, and 1 to append to current
-   $PATH.  */
-int set_exec_path(const char *path,int method)
+/* Replaces current $PATH */
+int set_exec_path(const char *path)
 {
-  char *p,*curpath=NULL;
-  size_t curlen=0;
+  char *p;
 
-  if(method==1 && (curpath=getenv("PATH")))
-    curlen=strlen(curpath)+1;
-
-  p=m_alloc(5+curlen+strlen(path)+1);
+  p=xmalloc(5+strlen(path)+1);
   strcpy(p,"PATH=");
-
-  if(curpath)
-    {
-      strcat(p,curpath);
-      strcat(p,":");
-    }
-
   strcat(p,path);
 
   if(DBG_EXTPROG)
-    log_debug("set_exec_path method %d: %s\n",method,p);
+    log_debug("set_exec_path: %s\n",p);
 
   /* Notice that path is never freed.  That is intentional due to the
      way putenv() works.  This leaks a few bytes if we call
@@ -131,17 +120,20 @@ static int make_tempdir(struct exec_info *info)
   char *tmp=opt.temp_dir,*namein=info->name,*nameout;
 
   if(!namein)
-    namein=info->binary?"tempin" EXTSEP_S "bin":"tempin" EXTSEP_S "txt";
+    namein=info->flags.binary?"tempin" EXTSEP_S "bin":"tempin" EXTSEP_S "txt";
 
-  nameout=info->binary?"tempout" EXTSEP_S "bin":"tempout" EXTSEP_S "txt";
+  nameout=info->flags.binary?"tempout" EXTSEP_S "bin":"tempout" EXTSEP_S "txt";
 
   /* Make up the temp dir and files in case we need them */
 
   if(tmp==NULL)
     {
 #if defined (_WIN32)
-      tmp=m_alloc(256);
-      if(GetTempPath(256,tmp)==0)
+      int err;
+
+      tmp=xmalloc(MAX_PATH);
+      err=GetTempPath(MAX_PATH,tmp);
+      if(err==0 || err>MAX_PATH)
 	strcpy(tmp,"c:\\windows\\temp");
       else
 	{
@@ -172,12 +164,12 @@ static int make_tempdir(struct exec_info *info)
 #endif
     }
 
-  info->tempdir=m_alloc(strlen(tmp)+strlen(DIRSEP_S)+10+1);
+  info->tempdir=xmalloc(strlen(tmp)+strlen(DIRSEP_S)+10+1);
 
   sprintf(info->tempdir,"%s" DIRSEP_S "gpg-XXXXXX",tmp);
 
 #if defined (_WIN32)
-  m_free(tmp);
+  xfree(tmp);
 #endif
 
   if(mkdtemp(info->tempdir)==NULL)
@@ -185,21 +177,21 @@ static int make_tempdir(struct exec_info *info)
 	      info->tempdir,strerror(errno));
   else
     {
-      info->madedir=1;
+      info->flags.madedir=1;
 
-      info->tempfile_in=m_alloc(strlen(info->tempdir)+
+      info->tempfile_in=xmalloc(strlen(info->tempdir)+
 				strlen(DIRSEP_S)+strlen(namein)+1);
       sprintf(info->tempfile_in,"%s" DIRSEP_S "%s",info->tempdir,namein);
 
-      if(!info->writeonly)
+      if(!info->flags.writeonly)
 	{
-	  info->tempfile_out=m_alloc(strlen(info->tempdir)+
+	  info->tempfile_out=xmalloc(strlen(info->tempdir)+
 				     strlen(DIRSEP_S)+strlen(nameout)+1);
 	  sprintf(info->tempfile_out,"%s" DIRSEP_S "%s",info->tempdir,nameout);
 	}
     }
 
-  return info->madedir?0:G10ERR_GENERAL;
+  return info->flags.madedir?0:G10ERR_GENERAL;
 }
 
 /* Expands %i and %o in the args to the full temp files within the
@@ -209,14 +201,14 @@ static int expand_args(struct exec_info *info,const char *args_in)
   const char *ch=args_in;
   unsigned int size,len;
 
-  info->use_temp_files=0;
-  info->keep_temp_files=0;
+  info->flags.use_temp_files=0;
+  info->flags.keep_temp_files=0;
 
   if(DBG_EXTPROG)
     log_debug("expanding string \"%s\"\n",args_in);
 
   size=100;
-  info->command=m_alloc(size);
+  info->command=xmalloc(size);
   len=0;
   info->command[0]='\0';
 
@@ -231,31 +223,31 @@ static int expand_args(struct exec_info *info,const char *args_in)
 	  switch(*ch)
 	    {
 	    case 'O':
-	      info->keep_temp_files=1;
+	      info->flags.keep_temp_files=1;
 	      /* fall through */
 
 	    case 'o': /* out */
-	      if(!info->madedir)
+	      if(!info->flags.madedir)
 		{
 		  if(make_tempdir(info))
 		    goto fail;
 		}
 	      append=info->tempfile_out;
-	      info->use_temp_files=1;
+	      info->flags.use_temp_files=1;
 	      break;
 
 	    case 'I':
-	      info->keep_temp_files=1;
+	      info->flags.keep_temp_files=1;
 	      /* fall through */
 
 	    case 'i': /* in */
-	      if(!info->madedir)
+	      if(!info->flags.madedir)
 		{
 		  if(make_tempdir(info))
 		    goto fail;
 		}
 	      append=info->tempfile_in;
-	      info->use_temp_files=1;
+	      info->flags.use_temp_files=1;
 	      break;
 
 	    case '%':
@@ -273,7 +265,7 @@ static int expand_args(struct exec_info *info,const char *args_in)
 		    applen=100;
 
 		  size+=applen;
-		  info->command=m_realloc(info->command,size);
+		  info->command=xrealloc(info->command,size);
 		}
 
 	      strcat(info->command,append);
@@ -285,7 +277,7 @@ static int expand_args(struct exec_info *info,const char *args_in)
 	  if(len==size-1) /* leave room for the \0 */
 	    {
 	      size+=100;
-	      info->command=m_realloc(info->command,size);
+	      info->command=xrealloc(info->command,size);
 	    }
 
 	  info->command[len++]=*ch;
@@ -296,14 +288,14 @@ static int expand_args(struct exec_info *info,const char *args_in)
     }
 
   if(DBG_EXTPROG)
-    log_debug("args expanded to \"%s\", use %d, keep %d\n",
-	      info->command,info->use_temp_files,info->keep_temp_files);
+    log_debug("args expanded to \"%s\", use %u, keep %u\n",info->command,
+	      info->flags.use_temp_files,info->flags.keep_temp_files);
 
   return 0;
 
  fail:
 
-  m_free(info->command);
+  xfree(info->command);
   info->command=NULL;
 
   return G10ERR_GENERAL;
@@ -338,19 +330,19 @@ int exec_write(struct exec_info **info,const char *program,
   if(program==NULL && args_in==NULL)
     BUG();
 
-  *info=m_alloc_clear(sizeof(struct exec_info));
+  *info=xmalloc_clear(sizeof(struct exec_info));
 
   if(name)
-    (*info)->name=m_strdup(name);
-  (*info)->binary=binary;
-  (*info)->writeonly=writeonly;
+    (*info)->name=xstrdup(name);
+  (*info)->flags.binary=binary;
+  (*info)->flags.writeonly=writeonly;
 
   /* Expand the args, if any */
   if(args_in && expand_args(*info,args_in))
     goto fail;
 
 #ifdef EXEC_TEMPFILE_ONLY
-  if(!(*info)->use_temp_files)
+  if(!(*info)->flags.use_temp_files)
     {
       log_error(_("this platform requires temporary files when calling"
 		  " external programs\n"));
@@ -361,7 +353,7 @@ int exec_write(struct exec_info **info,const char *program,
 
   /* If there are no args, or there are args, but no temp files, we
      can use fork/exec/pipe */
-  if(args_in==NULL || (*info)->use_temp_files==0)
+  if(args_in==NULL || (*info)->flags.use_temp_files==0)
     {
       int to[2],from[2];
 
@@ -395,7 +387,7 @@ int exec_write(struct exec_info **info,const char *program,
 
 	  /* If the program isn't going to respond back, they get to
              keep their stdout/stderr */
-	  if(!(*info)->writeonly)
+	  if(!(*info)->flags.writeonly)
 	    {
 	      /* implied close of STDERR */
 	      if(dup2(STDOUT_FILENO,STDERR_FILENO)==-1)
@@ -505,7 +497,7 @@ int exec_read(struct exec_info *info)
   fclose(info->tochild);
   info->tochild=NULL;
 
-  if(info->use_temp_files)
+  if(info->flags.use_temp_files)
     {
       if(DBG_EXTPROG)
 	log_debug("system() command is %s\n",info->command);
@@ -548,7 +540,7 @@ int exec_read(struct exec_info *info)
 	  goto fail;
 	}
 
-      if(!info->writeonly)
+      if(!info->flags.writeonly)
 	{
 	  info->fromchild=iobuf_open(info->tempfile_out);
           if (info->fromchild
@@ -601,7 +593,7 @@ int exec_finish(struct exec_info *info)
     }
 #endif
 
-  if(info->madedir && !info->keep_temp_files)
+  if(info->flags.madedir && !info->flags.keep_temp_files)
     {
       if(info->tempfile_in)
 	{
@@ -622,12 +614,12 @@ int exec_finish(struct exec_info *info)
 		 info->tempdir,strerror(errno));
     }
 
-  m_free(info->command);
-  m_free(info->name);
-  m_free(info->tempdir);
-  m_free(info->tempfile_in);
-  m_free(info->tempfile_out);
-  m_free(info);
+  xfree(info->command);
+  xfree(info->name);
+  xfree(info->tempdir);
+  xfree(info->tempfile_in);
+  xfree(info->tempfile_out);
+  xfree(info);
 
   return ret;
 }
