@@ -50,7 +50,6 @@
 #include "ttyio.h"
 #include "i18n.h"
 #include "status.h"
-#include "g10defs.h"
 #include "cardglue.h"
 
 
@@ -122,7 +121,7 @@ i18n_init(void)
 #else
 #ifdef ENABLE_NLS
     setlocale( LC_ALL, "" );
-    bindtextdomain( PACKAGE, G10_LOCALEDIR );
+    bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain( PACKAGE );
 #endif
 #endif
