@@ -1,6 +1,6 @@
 /* keygen.c - generate a key pair
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -60,6 +60,7 @@ enum para_name {
   pREVOKER,
   pUSERID,
   pEXPIREDATE,
+  pCREATETIME, /* in n seconds */
   pKEYEXPIRE, /* in n seconds */
   pSUBKEYEXPIRE, /* in n seconds */
   pPASSPHRASE,
@@ -78,6 +79,7 @@ struct para_data_s {
     union {
         DEK *dek;
         STRING2KEY *s2k;
+        u32 create;
         u32 expire;
         unsigned int usage;
         struct revocation_key revkey;
@@ -120,7 +122,8 @@ static int nzip_prefs;
 static int mdc_available,ks_modify;
 
 static void do_generate_keypair( struct para_data_s *para,
-				 struct output_control_s *outctrl, int card );
+				 struct output_control_s *outctrl,
+				 u32 timestamp, int card );
 static int  write_keyblock( IOBUF out, KBNODE node );
 static int gen_card_key (int algo, int keyno, int is_primary,
                          KBNODE pub_root, KBNODE sec_root,
@@ -777,7 +780,8 @@ make_backsig(PKT_signature *sig,PKT_public_key *pk,
 
   cache_public_key(sub_pk);
 
-  rc=make_keysig_packet(&backsig,pk,NULL,sub_pk,sub_sk,0x19,0,0,0,0,NULL,NULL);
+  rc=make_keysig_packet(&backsig,pk,NULL,sub_pk,sub_sk,0x19,0,0,
+			sub_pk->timestamp,0,NULL,NULL);
   if(rc)
     log_error("make_keysig_packet failed for backsig: %s\n",g10_errstr(rc));
   else
@@ -882,7 +886,7 @@ write_direct_sig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
     cache_public_key (pk);
 
     /* and make the signature */
-    rc = make_keysig_packet(&sig,pk,NULL,NULL,sk,0x1F,0,0,0,0,
+    rc = make_keysig_packet(&sig,pk,NULL,NULL,sk,0x1F,0,0,pk->timestamp,0,
 			    keygen_add_revkey,revkey);
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
@@ -926,8 +930,8 @@ write_selfsigs( KBNODE sec_root, KBNODE pub_root, PKT_secret_key *sk,
     cache_public_key (pk);
 
     /* and make the signature */
-    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0, 0, 0, 0,
-        		     keygen_add_std_prefs, pk );
+    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0, 0,
+			     pk->timestamp, 0, keygen_add_std_prefs, pk );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
 	return rc;
@@ -981,7 +985,8 @@ write_keybinding( KBNODE root, KBNODE pub_root,
     /* and make the signature */
     oduap.usage = use;
     oduap.pk = sub_pk;
-    rc=make_keysig_packet(&sig, pri_pk, NULL, sub_pk, pri_sk, 0x18, 0, 0, 0, 0,
+    rc=make_keysig_packet(&sig, pri_pk, NULL, sub_pk, pri_sk, 0x18, 0, 0,
+			  sub_pk->timestamp, 0,
 			  keygen_add_key_flags_and_expire, &oduap );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
@@ -1006,7 +1011,8 @@ write_keybinding( KBNODE root, KBNODE pub_root,
 
 static int
 gen_elg(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
-	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 expireval, int is_subkey)
+	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 timestamp,
+	u32 expireval, int is_subkey)
 {
     int rc;
     PACKET *pkt;
@@ -1035,11 +1041,11 @@ gen_elg(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 
     sk = xmalloc_clear( sizeof *sk );
     pk = xmalloc_clear( sizeof *pk );
-    sk->timestamp = pk->timestamp = make_timestamp();
+    sk->timestamp = pk->timestamp = timestamp;
     sk->version = pk->version = 4;
-    if( expireval ) {
-	sk->expiredate = pk->expiredate = sk->timestamp + expireval;
-    }
+    if( expireval )
+      sk->expiredate = pk->expiredate = sk->timestamp + expireval;
+
     sk->pubkey_algo = pk->pubkey_algo = algo;
 		       pk->pkey[0] = mpi_copy( skey[0] );
 		       pk->pkey[1] = mpi_copy( skey[1] );
@@ -1088,7 +1094,8 @@ gen_elg(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
  */
 static int
 gen_dsa(unsigned int nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
-	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 expireval, int is_subkey)
+	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 timestamp,
+	u32 expireval, int is_subkey)
 {
     int rc;
     PACKET *pkt;
@@ -1150,7 +1157,7 @@ gen_dsa(unsigned int nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 
     sk = xmalloc_clear( sizeof *sk );
     pk = xmalloc_clear( sizeof *pk );
-    sk->timestamp = pk->timestamp = make_timestamp();
+    sk->timestamp = pk->timestamp = timestamp;
     sk->version = pk->version = 4;
     if( expireval )
       sk->expiredate = pk->expiredate = sk->timestamp + expireval;
@@ -1209,7 +1216,8 @@ gen_dsa(unsigned int nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
  */
 static int
 gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
-	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 expireval, int is_subkey)
+	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 timestamp,
+	u32 expireval, int is_subkey)
 {
     int rc;
     PACKET *pkt;
@@ -1238,11 +1246,11 @@ gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 
     sk = xmalloc_clear( sizeof *sk );
     pk = xmalloc_clear( sizeof *pk );
-    sk->timestamp = pk->timestamp = make_timestamp();
+    sk->timestamp = pk->timestamp = timestamp;
     sk->version = pk->version = 4;
-    if( expireval ) {
-	sk->expiredate = pk->expiredate = sk->timestamp + expireval;
-    }
+    if( expireval )
+      sk->expiredate = pk->expiredate = sk->timestamp + expireval;
+
     sk->pubkey_algo = pk->pubkey_algo = algo;
 		       pk->pkey[0] = mpi_copy( skey[0] );
 		       pk->pkey[1] = mpi_copy( skey[1] );
@@ -1576,17 +1584,17 @@ ask_keysize( int algo )
  * similar.
  */
 u32
-parse_expire_string( const char *string )
+parse_expire_string(u32 timestamp,const char *string)
 {
     int mult;
-    u32 seconds,abs_date=0,curtime = make_timestamp();
+    u32 seconds,abs_date=0;
 
     if( !*string )
       seconds = 0;
     else if ( !strncmp (string, "seconds=", 8) )
       seconds = atoi (string+8);
-    else if( (abs_date = scan_isodatestr(string)) && abs_date > curtime )
-      seconds = abs_date - curtime;
+    else if( (abs_date = scan_isodatestr(string)) && abs_date > timestamp )
+      seconds = abs_date - timestamp;
     else if( (mult=check_valid_days(string)) )
       seconds = atoi(string) * 86400L * mult;
     else
@@ -1597,7 +1605,7 @@ parse_expire_string( const char *string )
 
 /* object == 0 for a key, and 1 for a sig */
 u32
-ask_expire_interval(int object,const char *def_expire)
+ask_expire_interval(u32 timestamp,int object,const char *def_expire)
 {
     u32 interval;
     char *answer;
@@ -1637,8 +1645,6 @@ ask_expire_interval(int object,const char *def_expire)
     answer = NULL;
     for(;;)
       {
-	u32 curtime=make_timestamp();
-
 	xfree(answer);
 	if(object==0)
 	  answer = cpr_get("keygen.valid",_("Key is valid for? (0) "));
@@ -1661,7 +1667,7 @@ ask_expire_interval(int object,const char *def_expire)
 	  }
 	cpr_kill_prompt();
 	trim_spaces(answer);
-	interval = parse_expire_string( answer );
+	interval = parse_expire_string( timestamp, answer );
 	if( interval == (u32)-1 )
 	  {
 	    tty_printf(_("invalid value\n"));
@@ -1679,11 +1685,11 @@ ask_expire_interval(int object,const char *def_expire)
 	    tty_printf(object==0
 		       ? _("Key expires at %s\n")
 		       : _("Signature expires at %s\n"),
-		       asctimestamp((ulong)(curtime + interval) ) );
+		       asctimestamp((ulong)(timestamp + interval) ) );
 	    /* FIXME: This check yields warning on alhas: Write a
 	       configure check and to this check here only for 32 bit
 	       machines */
-	    if( (time_t)((ulong)(curtime+interval)) < 0 )
+	    if( (time_t)((ulong)(timestamp+interval)) < 0 )
 	      tty_printf(_("Your system can't display dates beyond 2038.\n"
 			   "However, it will be correctly handled up to 2106.\n"));
 	  }
@@ -1697,14 +1703,6 @@ ask_expire_interval(int object,const char *def_expire)
     return interval;
 }
 
-u32
-ask_expiredate()
-{
-    u32 x = ask_expire_interval(0,NULL);
-    return x? make_timestamp() + x : 0;
-}
-
-
 static char *
 ask_user_id( int mode )
 {
@@ -1916,8 +1914,8 @@ do_ask_passphrase( STRING2KEY **ret_s2k )
 
 static int
 do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
-	   DEK *dek, STRING2KEY *s2k, PKT_secret_key **sk, u32 expiredate,
-	   int is_subkey )
+	   DEK *dek, STRING2KEY *s2k, PKT_secret_key **sk, u32 timestamp,
+	   u32 expiredate, int is_subkey )
 {
   int rc=0;
 
@@ -1929,14 +1927,14 @@ do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
 "generator a better chance to gain enough entropy.\n") );
 
   if( algo == PUBKEY_ALGO_ELGAMAL_E )
-    rc = gen_elg(algo, nbits, pub_root, sec_root, dek, s2k, sk, expiredate,
-		 is_subkey);
+    rc = gen_elg(algo, nbits, pub_root, sec_root, dek, s2k, sk, timestamp,
+		 expiredate, is_subkey);
   else if( algo == PUBKEY_ALGO_DSA )
-    rc = gen_dsa(nbits, pub_root, sec_root, dek, s2k, sk, expiredate,
-		 is_subkey);
+    rc = gen_dsa(nbits, pub_root, sec_root, dek, s2k, sk, timestamp,
+		 expiredate, is_subkey);
   else if( algo == PUBKEY_ALGO_RSA )
-    rc = gen_rsa(algo, nbits, pub_root, sec_root, dek, s2k, sk, expiredate,
-		 is_subkey);
+    rc = gen_rsa(algo, nbits, pub_root, sec_root, dek, s2k, sk, timestamp,
+		 expiredate, is_subkey);
   else
     BUG();
 
@@ -1958,7 +1956,7 @@ generate_user_id()
     if( !p )
 	return NULL;
     n = strlen(p);
-    uid = xmalloc_clear( sizeof *uid + n - 1 );
+    uid = xmalloc_clear( sizeof *uid + n );
     uid->len = n;
     strcpy(uid->name, p);
     uid->ref = 1;
@@ -2114,6 +2112,8 @@ get_parameter_u32( struct para_data_s *para, enum para_name key )
 	return r->u.expire;
     if( r->key == pKEYUSAGE || r->key == pSUBKEYUSAGE )
 	return r->u.usage;
+    if( r->key == pCREATETIME )
+	return r->u.create;
 
     return (unsigned int)strtoul( r->u.value, NULL, 10 );
 }
@@ -2154,6 +2154,13 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
   size_t n;
   char *p;
   int have_user_id=0,err,algo;
+  u32 timestamp;
+
+  /* If we were told a creation time from outside, use it.  Otherwise
+     look at the clock. */
+  timestamp=get_parameter_u32( para, pCREATETIME );
+  if(!timestamp)
+    timestamp=make_timestamp();
 
   /* Check that we have all required parameters. */
   r = get_parameter( para, pKEYTYPE );
@@ -2305,7 +2312,7 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
     {
       u32 seconds;
 
-      seconds = parse_expire_string( r->u.value );
+      seconds = parse_expire_string( timestamp, r->u.value );
       if( seconds == (u32)-1 )
 	{
 	  log_error("%s:%d: invalid expire date\n", fname, r->lnr );
@@ -2326,7 +2333,7 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
     return -1;
   }
 
-  do_generate_keypair( para, outctrl, card );
+  do_generate_keypair( para, outctrl, timestamp, card );
   return 0;
 }
 
@@ -2556,7 +2563,7 @@ generate_keypair (const char *fname, const char *card_serialno,
   int algo;
   unsigned int use;
   int both = 0;
-  u32 expire;
+  u32 timestamp,expire;
   struct para_data_s *para = NULL;
   struct para_data_s *r;
   struct output_control_s outctrl;
@@ -2576,6 +2583,13 @@ generate_keypair (const char *fname, const char *card_serialno,
       return;
     }
 
+  timestamp=make_timestamp();
+  r = xmalloc_clear( sizeof *r );
+  r->key = pCREATETIME;
+  r->u.create = timestamp;
+  r->next = para;
+  para = r;
+
   if (card_serialno)
     {
 #ifdef ENABLE_CARD_SUPPORT
@@ -2690,7 +2704,7 @@ generate_keypair (const char *fname, const char *card_serialno,
       para = r;
     }
    
-  expire = ask_expire_interval(0,NULL);
+  expire = ask_expire_interval(timestamp,0,NULL);
   r = xmalloc_clear( sizeof *r + 20 );
   r->key = pKEYEXPIRE;
   r->u.expire = expire;
@@ -2827,8 +2841,8 @@ start_tree(KBNODE *tree)
 }
 
 static void
-do_generate_keypair( struct para_data_s *para,
-		     struct output_control_s *outctrl, int card )
+do_generate_keypair( struct para_data_s *para,struct output_control_s *outctrl,
+		     u32 timestamp,int card )
 {
     KBNODE pub_root = NULL;
     KBNODE sec_root = NULL;
@@ -2931,6 +2945,7 @@ do_generate_keypair( struct para_data_s *para,
                         get_parameter_dek( para, pPASSPHRASE_DEK ),
                         get_parameter_s2k( para, pPASSPHRASE_S2K ),
                         &pri_sk,
+			timestamp,
                         get_parameter_u32( para, pKEYEXPIRE ), 0 );
       }
     else
@@ -2990,6 +3005,7 @@ do_generate_keypair( struct para_data_s *para,
                             get_parameter_dek( para, pPASSPHRASE_DEK ),
                             get_parameter_s2k( para, pPASSPHRASE_S2K ),
                             &sub_sk,
+			    timestamp,
                             get_parameter_u32( para, pSUBKEYEXPIRE ), 1 );
           }
         else
@@ -3151,7 +3167,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     char *passphrase = NULL;
     DEK *dek = NULL;
     STRING2KEY *s2k = NULL;
-    u32 cur_time;
+    u32 timestamp;
     int ask_pass = 0;
 
     /* break out the primary secret key */
@@ -3164,9 +3180,9 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     /* make a copy of the sk to keep the protected one in the keyblock */
     pri_sk = copy_secret_key( NULL, node->pkt->pkt.secret_key );
 
-    cur_time = make_timestamp();
-    if( pri_sk->timestamp > cur_time ) {
-	ulong d = pri_sk->timestamp - cur_time;
+    timestamp = make_timestamp();
+    if( pri_sk->timestamp > timestamp ) {
+	ulong d = pri_sk->timestamp - timestamp;
 	log_info( d==1 ? _("key has been created %lu second "
 			   "in future (time warp or clock problem)\n")
 		       : _("key has been created %lu seconds "
@@ -3215,7 +3231,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     algo = ask_algo( 1, &use );
     assert(algo);
     nbits = ask_keysize( algo );
-    expire = ask_expire_interval(0,NULL);
+    expire = ask_expire_interval(timestamp,0,NULL);
     if( !cpr_enabled() && !cpr_get_answer_is_yes("keygen.sub.okay",
 						  _("Really create? (y/N) ")))
 	goto leave;
@@ -3232,7 +3248,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     }
 
     rc = do_create( algo, nbits, pub_keyblock, sec_keyblock,
-		    dek, s2k, &sub_sk, expire, 1 );
+		    dek, s2k, &sub_sk, timestamp, expire, 1 );
     if( !rc )
 	rc = write_keybinding(pub_keyblock, pub_keyblock, pri_sk, sub_sk, use);
     if( !rc )
@@ -3269,9 +3285,8 @@ generate_card_subkeypair (KBNODE pub_keyblock, KBNODE sec_keyblock,
   PKT_secret_key *pri_sk = NULL, *sub_sk;
   int algo;
   unsigned int use;
-  u32 expire;
+  u32 timestamp,expire;
   char *passphrase = NULL;
-  u32 cur_time;
   struct para_data_s *para = NULL;
 
   assert (keyno >= 1 && keyno <= 3);
@@ -3291,10 +3306,10 @@ generate_card_subkeypair (KBNODE pub_keyblock, KBNODE sec_keyblock,
   /* Make a copy of the sk to keep the protected one in the keyblock */
   pri_sk = copy_secret_key (NULL, node->pkt->pkt.secret_key);
 
-  cur_time = make_timestamp();
-  if (pri_sk->timestamp > cur_time)
+  timestamp = make_timestamp();
+  if (pri_sk->timestamp > timestamp)
     {
-      ulong d = pri_sk->timestamp - cur_time;
+      ulong d = pri_sk->timestamp - timestamp;
       log_info (d==1 ? _("key has been created %lu second "
                          "in future (time warp or clock problem)\n")
                      : _("key has been created %lu seconds "
@@ -3333,7 +3348,7 @@ generate_card_subkeypair (KBNODE pub_keyblock, KBNODE sec_keyblock,
     goto leave;
 
   algo = PUBKEY_ALGO_RSA;
-  expire = ask_expire_interval (0,NULL);
+  expire = ask_expire_interval (timestamp,0,NULL);
   if (keyno == 1)
     use = PUBKEY_USAGE_SIG;
   else if (keyno == 2)
