@@ -1,6 +1,6 @@
 /* misc.c -  miscellaneous functions
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+ *               2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -289,7 +289,7 @@ checksum_mpi( MPI a )
     nbits = mpi_get_nbits(a);
     csum = checksum_u16( nbits );
     csum += checksum( buffer, nbytes );
-    m_free( buffer );
+    xfree( buffer );
     return csum;
 }
 
@@ -407,19 +407,19 @@ openpgp_pk_algo_usage ( int algo )
     /* they are hardwired in gpg 1.0 */
     switch ( algo ) {    
       case PUBKEY_ALGO_RSA:
-          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC | PUBKEY_USAGE_AUTH;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC | PUBKEY_USAGE_AUTH;
           break;
       case PUBKEY_ALGO_RSA_E:
           use = PUBKEY_USAGE_ENC;
           break;
       case PUBKEY_ALGO_RSA_S:
-          use = PUBKEY_USAGE_SIG;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG;
           break;
       case PUBKEY_ALGO_ELGAMAL_E:
           use = PUBKEY_USAGE_ENC;
           break;
       case PUBKEY_ALGO_DSA:  
-          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_AUTH;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG | PUBKEY_USAGE_AUTH;
           break;
       default:
           break;
@@ -445,8 +445,8 @@ idea_cipher_warn(int show)
   if(!warned || show)
     {
       log_info(_("the IDEA cipher plugin is not present\n"));
-      log_info(_("please see http://www.gnupg.org/why-not-idea.html "
-		 "for more information\n"));
+      log_info(_("please see %s for more information\n"),
+               "http://www.gnupg.org/faq/why-not-idea.html");
       warned=1;
     }
 }
@@ -468,7 +468,7 @@ static unsigned long get_signature_count(PKT_secret_key *sk)
   return 0;
 }
 
-/* Expand %-strings.  Returns a string which must be m_freed.  Returns
+/* Expand %-strings.  Returns a string which must be xfreed.  Returns
    NULL if the string cannot be expanded (too large). */
 char *
 pct_expando(const char *string,struct expando_args *args)
@@ -500,7 +500,7 @@ pct_expando(const char *string,struct expando_args *args)
 	    goto fail;
 
 	  maxlen+=1024;
-	  ret=m_realloc(ret,maxlen);
+	  ret=xrealloc(ret,maxlen);
 	}
 
       done=0;
@@ -571,7 +571,7 @@ pct_expando(const char *string,struct expando_args *args)
 		    else if(args->sk->main_keyid[0] || args->sk->main_keyid[1])
 		      {
 			PKT_public_key *pk=
-			  m_alloc_clear(sizeof(PKT_public_key));
+			  xmalloc_clear(sizeof(PKT_public_key));
 
 			if(get_pubkey_fast(pk,args->sk->main_keyid)==0)
 			  fingerprint_from_pk(pk,array,&len);
@@ -662,7 +662,7 @@ pct_expando(const char *string,struct expando_args *args)
   return ret;
 
  fail:
-  m_free(ret);
+  xfree(ret);
   return NULL;
 }
 
@@ -966,6 +966,24 @@ parse_options(char *str,unsigned int *options,
 {
   char *tok;
 
+  if (str && !strcmp (str, "help"))
+    {
+      int i,maxlen=0;
+
+      /* Figure out the longest option name so we can line these up
+	 neatly. */
+      for(i=0;opts[i].name;i++)
+	if(opts[i].help && maxlen<strlen(opts[i].name))
+	  maxlen=strlen(opts[i].name);
+
+      for(i=0;opts[i].name;i++)
+        if(opts[i].help)
+	  printf("%s%*s%s\n",opts[i].name,
+		 maxlen+2-(int)strlen(opts[i].name),"",_(opts[i].help));
+
+      g10_exit(0);
+    }
+
   while((tok=optsep(&str)))
     {
       int i,rev=0;
@@ -1063,6 +1081,42 @@ unescape_percent_string (const unsigned char *s)
 }
 
 
+int
+has_invalid_email_chars (const char *s)
+{
+  int at_seen=0;
+  const char *valid_chars=
+    "01234567890_-.abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+  for ( ; *s; s++ ) 
+    {
+      if ( *s & 0x80 )
+        return 1;
+      if ( *s == '@' )
+        at_seen=1;
+      else if ( !at_seen && !( !!strchr( valid_chars, *s ) || *s == '+' ) )
+        return 1;
+      else if ( at_seen && !strchr( valid_chars, *s ) )
+        return 1;
+    }
+  return 0;
+}
+
+
+/* Check whether NAME represents a valid mailbox according to
+   RFC822. Returns true if so. */
+int
+is_valid_mailbox (const char *name)
+{
+  return !( !name
+            || !*name
+            || has_invalid_email_chars (name)
+            || string_count_chr (name,'@') != 1
+            || *name == '@'
+            || name[strlen(name)-1] == '@'
+            || name[strlen(name)-1] == '.'
+            || strstr (name, "..") );
+}
 
 
 /* This is a helper function to load a Windows function from either of
@@ -1184,3 +1238,47 @@ get_libexecdir (void)
 
   return GNUPG_LIBEXECDIR;
 }
+
+/* Similar to access(2), but uses PATH to find the file. */
+int
+path_access(const char *file,int mode)
+{
+  char *envpath;
+  int ret=-1;
+
+  envpath=getenv("PATH");
+
+  if(!envpath
+#ifdef HAVE_DRIVE_LETTERS
+     || (((file[0]>='A' && file[0]<='Z')
+	  || (file[0]>='a' && file[0]<='z'))
+	 && file[1]==':')
+#else
+     || file[0]=='/'
+#endif
+     )
+    return access(file,mode);
+  else
+    {
+      /* At least as large as, but most often larger than we need. */
+      char *buffer=xmalloc(strlen(envpath)+1+strlen(file)+1);
+      char *split,*item,*path=xstrdup(envpath);
+
+      split=path;
+
+      while((item=strsep(&split,PATHSEP_S)))
+	{
+	  strcpy(buffer,item);
+	  strcat(buffer,"/");
+	  strcat(buffer,file);
+	  ret=access(buffer,mode);
+	  if(ret==0)
+	    break;
+	}
+
+      xfree(path);
+      xfree(buffer);
+    }
+
+  return ret;
+}
