@@ -144,8 +144,8 @@ ask_outfile_name( const char *name, size_t namelen )
 
     s = _("Enter new filename");
 
-    n = strlen(s) + namelen + 10;
     defname = name && namelen? make_printable_string( name, namelen, 0): NULL;
+    n = strlen(s) + (defname?strlen (defname):0) + 10;
     prompt = xmalloc(n);
     if( defname )
 	sprintf(prompt, "%s [%s]: ", s, defname );
@@ -345,7 +345,7 @@ copy_options_file( const char *destdir )
         errno = EPERM;
       }
     if( !src ) {
-	log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
+	log_info (_("can't open `%s': %s\n"), fname, strerror(errno) );
 	xfree(fname);
 	return;
     }
@@ -360,7 +360,7 @@ copy_options_file( const char *destdir )
       dst = fopen( fname, "w" );
     umask(oldmask);
     if( !dst ) {
-	log_error(_("can't create `%s': %s\n"), fname, strerror(errno) );
+	log_info (_("can't create `%s': %s\n"), fname, strerror(errno) );
 	fclose( src );
 	xfree(fname);
 	return;
