@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -34,6 +35,7 @@
 #include "options.h"
 #include "main.h"
 #include "i18n.h"
+#include "cardglue.h"
 
 static int get_it( PKT_pubkey_enc *k,
 		   DEK *dek, PKT_secret_key *sk, u32 *keyid );
@@ -77,7 +79,7 @@ get_session_key( PKT_pubkey_enc *k, DEK *dek )
 	goto leave;
 
     if( (k->keyid[0] || k->keyid[1]) && !opt.try_all_secrets ) {
-	sk = m_alloc_clear( sizeof *sk );
+	sk = xmalloc_clear( sizeof *sk );
 	sk->pubkey_algo = k->pubkey_algo; /* we want a pubkey with this algo*/
 	if( !(rc = get_seckey( sk, k->keyid )) )
 	    rc = get_it( k, dek, sk, k->keyid );
@@ -85,12 +87,13 @@ get_session_key( PKT_pubkey_enc *k, DEK *dek )
     else { /* anonymous receiver: Try all available secret keys */
 	void *enum_context = NULL;
 	u32 keyid[2];
+	char *p;
 
 	for(;;) {
 	    if( sk )
 		free_secret_key( sk );
-	    sk = m_alloc_clear( sizeof *sk );
-	    rc=enum_secret_keys( &enum_context, sk, 1);
+	    sk = xmalloc_clear( sizeof *sk );
+	    rc=enum_secret_keys( &enum_context, sk, 1, 0);
 	    if( rc ) {
 		rc = G10ERR_NO_SECKEY;
 		break;
@@ -98,17 +101,42 @@ get_session_key( PKT_pubkey_enc *k, DEK *dek )
 	    if( sk->pubkey_algo != k->pubkey_algo )
 		continue;
 	    keyid_from_sk( sk, keyid );
-	    log_info(_("anonymous recipient; trying secret key %08lX ...\n"),
-                     (ulong)keyid[1] );
-	    rc = check_secret_key( sk, 1 ); /* ask only once */
+	    log_info(_("anonymous recipient; trying secret key %s ...\n"),
+                     keystr(keyid));
+
+	    if(!opt.try_all_secrets && !is_status_enabled())
+	      {
+		p=get_last_passphrase();
+		set_next_passphrase(p);
+		xfree(p);
+	      }
+
+	    rc = check_secret_key( sk, opt.try_all_secrets?1:-1 ); /* ask
+								      only
+								      once */
 	    if( !rc )
+	      {
 		rc = get_it( k, dek, sk, keyid );
-	    if( !rc ) {
+		/* Successfully checked the secret key (either it was
+		   a card, had no passphrase, or had the right
+		   passphrase) but couldn't decrypt the session key,
+		   so thus that key is not the anonymous recipient.
+		   Move the next passphrase into last for the next
+		   round.  We only do this if the secret key was
+		   successfully checked as in the normal case,
+		   check_secret_key handles this for us via
+		   passphrase_to_dek */
+		if(rc)
+		  next_to_last_passphrase();
+	      }
+
+	    if( !rc )
+	      {
 		log_info(_("okay, we are the anonymous recipient.\n") );
 		break;
-	    }
+	      }
 	}
-	enum_secret_keys( &enum_context, NULL, 0 ); /* free context */
+	enum_secret_keys( &enum_context, NULL, 0, 0 ); /* free context */
     }
 
   leave:
@@ -121,17 +149,51 @@ get_session_key( PKT_pubkey_enc *k, DEK *dek )
 static int
 get_it( PKT_pubkey_enc *enc, DEK *dek, PKT_secret_key *sk, u32 *keyid )
 {
-    int rc;
-    MPI plain_dek  = NULL;
-    byte *frame = NULL;
-    unsigned n, nframe;
-    u16 csum, csum2;
+  int rc;
+  MPI plain_dek  = NULL;
+  byte *frame = NULL;
+  unsigned n, nframe;
+  u16 csum, csum2;
+  
+  int card = 0;
 
-    rc = pubkey_decrypt(sk->pubkey_algo, &plain_dek, enc->data, sk->skey );
-    if( rc )
+  if (sk->is_protected && sk->protect.s2k.mode == 1002)
+    { /* Note, that we only support RSA for now. */
+#ifdef ENABLE_CARD_SUPPORT
+      unsigned char *rbuf;
+      size_t rbuflen;
+      char *snbuf;
+      unsigned char *indata = NULL;
+      unsigned int indatalen;
+
+      snbuf = serialno_and_fpr_from_sk (sk->protect.iv, sk->protect.ivlen, sk);
+
+      indata = mpi_get_buffer (enc->data[0], &indatalen, NULL);
+      if (!indata)
+        BUG ();
+
+      rc = agent_scd_pkdecrypt (snbuf, indata, indatalen, &rbuf, &rbuflen);
+      xfree (snbuf);
+      xfree (indata);
+      if (rc)
+        goto leave;
+
+      frame = rbuf;
+      nframe = rbuflen;
+      card = 1;
+#else
+      rc = G10ERR_UNSUPPORTED;
+      goto leave;
+#endif /*!ENABLE_CARD_SUPPORT*/
+    }
+  else
+    {
+      rc = pubkey_decrypt(sk->pubkey_algo, &plain_dek, enc->data, sk->skey );
+      if( rc )
 	goto leave;
-    frame = mpi_get_buffer( plain_dek, &nframe, NULL );
-    mpi_free( plain_dek ); plain_dek = NULL;
+      frame = mpi_get_buffer( plain_dek, &nframe, NULL );
+      mpi_free( plain_dek ); plain_dek = NULL;
+    }
 
     /* Now get the DEK (data encryption key) from the frame
      *
@@ -153,18 +215,22 @@ get_it( PKT_pubkey_enc *enc, DEK *dek, PKT_secret_key *sk, u32 *keyid )
     if( DBG_CIPHER )
 	log_hexdump("DEK frame:", frame, nframe );
     n=0;
-    if( n + 7 > nframe )
-	{ rc = G10ERR_WRONG_SECKEY; goto leave; }
-    if( frame[n] == 1 && frame[nframe-1] == 2 ) {
-	log_info(_("old encoding of the DEK is not supported\n"));
-	rc = G10ERR_CIPHER_ALGO;
-	goto leave;
-    }
-    if( frame[n] != 2 )  /* somethink is wrong */
-	{ rc = G10ERR_WRONG_SECKEY; goto leave; }
-    for(n++; n < nframe && frame[n]; n++ ) /* skip the random bytes */
-	;
-    n++; /* and the zero byte */
+    if (!card)
+      {
+        if( n + 7 > nframe )
+          { rc = G10ERR_WRONG_SECKEY; goto leave; }
+        if( frame[n] == 1 && frame[nframe-1] == 2 ) {
+          log_info(_("old encoding of the DEK is not supported\n"));
+          rc = G10ERR_CIPHER_ALGO;
+          goto leave;
+        }
+        if( frame[n] != 2 )  /* somethink is wrong */
+          { rc = G10ERR_WRONG_SECKEY; goto leave; }
+        for(n++; n < nframe && frame[n]; n++ ) /* skip the random bytes */
+          ;
+        n++; /* and the zero byte */
+      }
+
     if( n + 4 > nframe )
 	{ rc = G10ERR_WRONG_SECKEY; goto leave; }
 
@@ -209,13 +275,12 @@ get_it( PKT_pubkey_enc *enc, DEK *dek, PKT_secret_key *sk, u32 *keyid )
             rc = -1;
 	    log_error("oops: public key not found for preference check\n");
         }
-	else if( pkb->pkt->pkt.public_key->selfsigversion > 3
-		 && dek->algo != CIPHER_ALGO_3DES
-		 && !opt.quiet
-		 && !is_algo_in_prefs( pkb, PREFTYPE_SYM, dek->algo ))
-	  log_info(_("NOTE: cipher algorithm %d not found in preferences\n"),
-		   dek->algo );
-
+	else if(pkb->pkt->pkt.public_key->selfsigversion > 3
+		&& dek->algo != CIPHER_ALGO_3DES
+		&& !opt.quiet
+		&& !is_algo_in_prefs( pkb, PREFTYPE_SYM, dek->algo ))
+	  log_info(_("WARNING: cipher algorithm %s not found in recipient"
+		     " preferences\n"),cipher_algo_to_string(dek->algo));
         if (!rc) {
             KBNODE k;
             
@@ -234,12 +299,12 @@ get_it( PKT_pubkey_enc *enc, DEK *dek, PKT_secret_key *sk, u32 *keyid )
             if (!pk)
                 BUG ();
             if ( pk->expiredate && pk->expiredate <= make_timestamp() ) {
-                log_info(_("NOTE: secret key %08lX expired at %s\n"),
-                         (ulong)keyid[1], asctimestamp( pk->expiredate) );
+                log_info(_("NOTE: secret key %s expired at %s\n"),
+                         keystr(keyid), asctimestamp( pk->expiredate) );
             }
         }
 
-        if ( pk && pk->is_revoked ) {
+        if ( pk &&  pk->is_revoked ) {
             log_info( _("NOTE: key has been revoked") );
             putc( '\n', log_stream() );
             show_revocation_reason( pk, 1 );
@@ -252,7 +317,7 @@ get_it( PKT_pubkey_enc *enc, DEK *dek, PKT_secret_key *sk, u32 *keyid )
 
   leave:
     mpi_free(plain_dek);
-    m_free(frame);
+    xfree(frame);
     return rc;
 }
 
