@@ -1,5 +1,6 @@
 /* revoke.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -59,15 +61,15 @@ revocation_reason_build_cb( PKT_signature *sig, void *opaque )
 	ud = native_to_utf8( reason->desc );
 	buflen += strlen(ud);
     }
-    buffer = m_alloc( buflen );
+    buffer = xmalloc( buflen );
     *buffer = reason->code;
     if( ud ) {
 	memcpy(buffer+1, ud, strlen(ud) );
-	m_free( ud );
+	xfree( ud );
     }
 
     build_sig_subpkt( sig, SIGSUBPKT_REVOC_REASON, buffer, buflen );
-    m_free( buffer );
+    xfree( buffer );
     return 0;
 }
 
@@ -89,7 +91,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
   node=find_kbnode(keyblock,PKT_PUBLIC_KEY);
   if(!node)
     {
-      log_error(_("key incomplete\n"));
+      log_error("key incomplete\n");
       return G10ERR_GENERAL;
     }
 
@@ -142,7 +144,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
 	    break;
 	  else
 	    {
-	      log_error(_("key %08lX incomplete\n"),(ulong)keyid[1]);
+	      log_error(_("key %s has no user IDs\n"),keystr(keyid));
 	      return G10ERR_GENERAL;
 	    }
 	}
@@ -195,7 +197,7 @@ export_minimal_pk(IOBUF out,KBNODE keyblock,
  * Generate a revocation certificate for UNAME via a designated revoker
  */
 int
-gen_desig_revoke( const char *uname )
+gen_desig_revoke( const char *uname, STRLIST locusr )
 {
     int rc = 0;
     armor_filter_context_t afx;
@@ -209,11 +211,13 @@ gen_desig_revoke( const char *uname )
     KBNODE keyblock=NULL,node;
     u32 keyid[2];
     int i,any=0;
+    SK_LIST sk_list=NULL;
 
-    if( opt.batch ) {
-	log_error(_("sorry, can't do this in batch mode\n"));
+    if( opt.batch )
+      {
+	log_error(_("can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-    }
+      }
 
     memset( &afx, 0, sizeof afx);
 
@@ -221,7 +225,7 @@ gen_desig_revoke( const char *uname )
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
     if (rc) {
-	log_error (_("key `%s' not found: %s\n"),uname, g10_errstr (rc));
+	log_error (_("key \"%s\" not found: %s\n"),uname, g10_errstr (rc));
 	goto leave;
     }
 
@@ -243,6 +247,13 @@ gen_desig_revoke( const char *uname )
 
     keyid_from_pk(pk,keyid);
 
+    if(locusr)
+      {
+	rc=build_sk_list(locusr,&sk_list,0,PUBKEY_USAGE_CERT);
+	if(rc)
+	  goto leave;
+      }
+
     /* Are we a designated revoker for this key? */
 
     if(!pk->revkey && pk->numrevkeys)
@@ -250,51 +261,59 @@ gen_desig_revoke( const char *uname )
 
     for(i=0;i<pk->numrevkeys;i++)
       {
+	SK_LIST list;
+
 	if(sk)
 	  free_secret_key(sk);
 
-	sk=m_alloc_clear(sizeof(*sk));
+	if(sk_list)
+	  {
+	    for(list=sk_list;list;list=list->next)
+	      {
+		byte fpr[MAX_FINGERPRINT_LEN];
+		size_t fprlen;
 
-	rc=get_seckey_byfprint(sk,pk->revkey[i].fpr,MAX_FINGERPRINT_LEN);
+		fingerprint_from_sk(list->sk,fpr,&fprlen);
+
+		/* Don't get involved with keys that don't have 160
+		   bit fingerprints */
+		if(fprlen!=20)
+		  continue;
+
+		if(memcmp(fpr,pk->revkey[i].fpr,20)==0)
+		  break;
+	      }
+
+	    if(list)
+	      sk=copy_secret_key(NULL,list->sk);
+	    else
+	      continue;
+	  }
+	else
+	  {
+	    sk=xmalloc_secure_clear(sizeof(*sk));
+	    rc=get_seckey_byfprint(sk,pk->revkey[i].fpr,MAX_FINGERPRINT_LEN);
+	  }
 
 	/* We have the revocation key */
 	if(!rc)
 	  {
-	    size_t n;
-	    char *p;
-	    u32 sk_keyid[2];
-	    PKT_signature *revkey=NULL;
+	    PKT_signature *revkey = NULL;
 
-	    any=1;
-	    keyid_from_sk(sk,sk_keyid);
+	    any = 1;
 
-	    tty_printf("\npub  %4u%c/%08lX %s   ",
-		       nbits_from_pk( pk ),
-		       pubkey_letter( pk->pubkey_algo ),
-		       (ulong)keyid[1], datestr_from_pk(pk) );
+            print_pubkey_info (NULL, pk);
+	    tty_printf ("\n");
 
-	    p = get_user_id( keyid, &n );
-	    tty_print_utf8_string( p, n );
-	    m_free(p);
-	    tty_printf("\n\n");
+	    tty_printf (_("To be revoked by:\n"));
+            print_seckey_info (sk);
 
-	    tty_printf(_("To be revoked by:\n"));
-
-	    tty_printf("\nsec  %4u%c/%08lX %s   ",
-		       nbits_from_sk( sk ),
-		       pubkey_letter( sk->pubkey_algo ),
-		       (ulong)sk_keyid[1], datestr_from_sk(sk) );
-
-	    p = get_user_id( sk_keyid, &n );
-	    tty_print_utf8_string( p, n );
-	    m_free(p);
-	    tty_printf("\n");
 	    if(pk->revkey[i].class&0x40)
 	      tty_printf(_("(This is a sensitive revocation key)\n"));
 	    tty_printf("\n");
 
 	    if( !cpr_get_answer_is_yes("gen_desig_revoke.okay",
-		       _("Create a revocation certificate for this key? ")) )
+         _("Create a designated revocation certificate for this key? (y/N) ")))
 	      continue;
 
 	    /* get the reason for the revocation (this is always v4) */
@@ -313,7 +332,8 @@ gen_desig_revoke( const char *uname )
 	      goto leave;
 
 	    afx.what = 1;
-	    afx.hdrlines = "Comment: A designated revocation certificate should follow\n";
+	    afx.hdrlines = "Comment: A designated revocation certificate"
+	      " should follow\n";
 	    iobuf_push_filter( out, armor_filter, &afx );
 
 	    /* create it */
@@ -390,7 +410,7 @@ gen_desig_revoke( const char *uname )
       }
 
     if(!any)
-      log_error(_("no revocation keys found for `%s'\n"),uname);
+      log_error(_("no revocation keys found for \"%s\"\n"),uname);
 
   leave:
     if( pk )
@@ -400,6 +420,8 @@ gen_desig_revoke( const char *uname )
     if( sig )
 	free_seckey_enc( sig );
 
+    release_sk_list(sk_list);
+
     if( rc )
 	iobuf_cancel(out);
     else
@@ -429,10 +451,11 @@ gen_revoke( const char *uname )
     struct revocation_reason_info *reason = NULL;
     KEYDB_SEARCH_DESC desc;
 
-    if( opt.batch ) {
-	log_error(_("sorry, can't do this in batch mode\n"));
+    if( opt.batch )
+      {
+	log_error(_("can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
-    }
+      }
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
@@ -443,11 +466,12 @@ gen_revoke( const char *uname )
     kdbhd = keydb_new (1);
     classify_user_id (uname, &desc);
     rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
-    if (rc) {
-	log_error (_("secret key `%s' not found: %s\n"),
+    if (rc)
+      {
+	log_error (_("secret key \"%s\" not found: %s\n"),
                    uname, g10_errstr (rc));
 	goto leave;
-    }
+      }
 
     rc = keydb_get_keyblock (kdbhd, &keyblock );
     if( rc ) {
@@ -464,18 +488,9 @@ gen_revoke( const char *uname )
      * it's used all over the source */
     sk = node->pkt->pkt.secret_key;
     keyid_from_sk( sk, sk_keyid );
-    tty_printf("\nsec  %4u%c/%08lX %s   ",
-	      nbits_from_sk( sk ),
-	      pubkey_letter( sk->pubkey_algo ),
-	      (ulong)sk_keyid[1], datestr_from_sk(sk) );
-    {
-	size_t n;
-	char *p = get_user_id( sk_keyid, &n );
-	tty_print_utf8_string( p, n );
-	m_free(p);
-	tty_printf("\n");
-    }
-    pk = m_alloc_clear( sizeof *pk );
+    print_seckey_info (sk);
+
+    pk = xmalloc_clear( sizeof *pk );
 
     /* FIXME: We should get the public key direct from the secret one */
 
@@ -500,10 +515,11 @@ gen_revoke( const char *uname )
 
     tty_printf("\n");
     if( !cpr_get_answer_is_yes("gen_revoke.okay",
-			_("Create a revocation certificate for this key? ")) ){
+		  _("Create a revocation certificate for this key? (y/N) ")) )
+      {
 	rc = 0;
 	goto leave;
-    }
+      }
 
     if(sk->version>=4 || opt.force_v4_certs) {
       /* get the reason for the revocation */
@@ -519,11 +535,15 @@ gen_revoke( const char *uname )
 	log_error(_("unknown protection algorithm\n"));
 	rc = G10ERR_PUBKEY_ALGO;
 	break;
+      case -3:
+	tty_printf (_("Secret parts of primary key are not available.\n"));
+        rc = G10ERR_NO_SECKEY;
+        break;
       case 0:
 	tty_printf(_("NOTE: This key is not protected!\n"));
 	break;
       default:
-	rc = check_secret_key( sk, 0 );
+        rc = check_secret_key( sk, 0 );
 	break;
     }
     if( rc )
@@ -609,7 +629,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 
     do {
         code=-1;
-	m_free(description);
+	xfree(description);
 	description = NULL;
 
 	tty_printf(_("Please select the reason for the revocation:\n"));
@@ -640,7 +660,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
  	        n = -1;
 	    else
 		n = atoi(answer);
-	    m_free(answer);
+	    xfree(answer);
 	    if( n == 0 ) {
 	        code = 0x00; /* no particular reason */
 		code_text = text_0;
@@ -672,25 +692,25 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    trim_trailing_ws( answer, strlen(answer) );
 	    cpr_kill_prompt();
 	    if( !*answer ) {
-		m_free(answer);
+		xfree(answer);
 		break;
 	    }
 
 	    {
 		char *p = make_printable_string( answer, strlen(answer), 0 );
-		m_free(answer);
+		xfree(answer);
 		answer = p;
 	    }
 
 	    if( !description )
-		description = m_strdup(answer);
+		description = xstrdup(answer);
 	    else {
-		char *p = m_alloc( strlen(description) + strlen(answer) + 2 );
+		char *p = xmalloc( strlen(description) + strlen(answer) + 2 );
 		strcpy(stpcpy(stpcpy( p, description),"\n"),answer);
-		m_free(description);
+		xfree(description);
 		description = p;
 	    }
-	    m_free(answer);
+	    xfree(answer);
 	}
 
 	tty_printf(_("Reason for revocation: %s\n"), code_text );
@@ -700,9 +720,9 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 	    tty_printf("%s\n", description );
 
     } while( !cpr_get_answer_is_yes("ask_revocation_reason.okay",
-					    _("Is this okay? "))  );
+					    _("Is this okay? (y/N) "))  );
 
-    reason = m_alloc( sizeof *reason );
+    reason = xmalloc( sizeof *reason );
     reason->code = code;
     reason->desc = description;
     return reason;
@@ -712,7 +732,7 @@ void
 release_revocation_reason_info( struct revocation_reason_info *reason )
 {
     if( reason ) {
-	m_free( reason->desc );
-	m_free( reason );
+	xfree( reason->desc );
+	xfree( reason );
     }
 }
