@@ -308,8 +308,7 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
         xfree (snbuf);
         if (!rc)
           {
-            sig->data[0] = mpi_alloc ( (rbuflen+BYTES_PER_MPI_LIMB-1)
-                                       / BYTES_PER_MPI_LIMB );
+            sig->data[0] = mpi_alloc ( mpi_nlimb_hint_from_nbytes (rbuflen) );
             mpi_set_buffer (sig->data[0], rbuf, rbuflen, 0);
             xfree (rbuf);
           }
@@ -573,7 +572,8 @@ write_onepass_sig_packets (SK_LIST sk_list, IOBUF out, int sigclass )
  * Helper to write the plaintext (literal data) packet
  */
 static int
-write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
+write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname,
+			int ptmode, u32 timestamp)
 {
     PKT_plaintext *pt = NULL;
     u32 filesize;
@@ -614,7 +614,7 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
     if (!opt.no_literal) {
         PACKET pkt;
 
-        pt->timestamp = make_timestamp ();
+        pt->timestamp = timestamp;
         pt->mode = ptmode;
         pt->len = filesize;
         pt->new_ctb = !pt->len && !RFC1991;
@@ -750,7 +750,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     SK_LIST sk_list = NULL;
     SK_LIST sk_rover = NULL;
     int multifile = 0;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -775,9 +775,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
@@ -1027,7 +1027,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     }
     else {
         rc = write_plaintext_packet (out, inp, fname,
-                                     opt.textmode && !outfile ? 't':'b');
+                                     opt.textmode && !outfile ? 't':'b',
+				     create_time);
     }
 
     /* catch errors from above */
@@ -1037,7 +1038,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     /* write the signatures */
     rc = write_signature_packets (sk_list, out, mfx.md,
                                   opt.textmode && !outfile? 0x01 : 0x00,
-				  0, duration, detached ? 'D':'S');
+				  create_time, duration, detached ? 'D':'S');
     if( rc )
         goto leave;
 
@@ -1076,7 +1077,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     SK_LIST sk_rover = NULL;
     int old_style = RFC1991;
     int only_md5 = 0;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
@@ -1084,9 +1085,9 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
@@ -1196,7 +1197,8 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     iobuf_push_filter( out, armor_filter, &afx );
 
     /* write the signatures */
-    rc=write_signature_packets (sk_list, out, textmd, 0x01, 0, duration, 'C');
+    rc=write_signature_packets (sk_list, out, textmd, 0x01,
+				create_time, duration, 'C');
     if( rc )
         goto leave;
 
@@ -1231,7 +1233,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     SK_LIST sk_list = NULL;
     SK_LIST sk_rover = NULL;
     int algo;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -1243,9 +1245,9 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     rc = build_sk_list (locusr, &sk_list, 1, PUBKEY_USAGE_SIG);
@@ -1349,7 +1351,8 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* Pipe data through all filters; i.e. write the signed stuff */
     /*(current filters: zip - encrypt - armor)*/
-    rc = write_plaintext_packet (out, inp, fname, opt.textmode ? 't':'b');
+    rc = write_plaintext_packet (out, inp, fname, opt.textmode ? 't':'b',
+				 create_time);
     if (rc)
 	goto leave;
     
@@ -1357,7 +1360,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     /*(current filters: zip - encrypt - armor)*/
     rc = write_signature_packets (sk_list, out, mfx.md,
 				  opt.textmode? 0x01 : 0x00,
-				  0, duration, 'S');
+				  create_time, duration, 'S');
     if( rc )
         goto leave;
 
