@@ -44,7 +44,7 @@ release_sk_list( SK_LIST sk_list )
     for( ; sk_list; sk_list = sk_rover ) {
 	sk_rover = sk_list->next;
 	free_secret_key( sk_list->sk );
-	m_free( sk_list );
+	xfree( sk_list );
     }
 }
 
@@ -69,7 +69,8 @@ is_insecure( PKT_secret_key *sk )
                 continue; /* skip attribute packets */
             if ( strstr( id->name, "(insecure!)" )
                  || strstr( id->name, "not secure" )
-                 || strstr( id->name, "do not use" ) ) {
+                 || strstr( id->name, "do not use" )
+                 || strstr( id->name, "(INSECURE!)" ) ) {
                 insecure = 1;
                 break;
             }
@@ -112,7 +113,7 @@ build_sk_list( STRLIST locusr, SK_LIST *ret_sk_list,
       { /* use the default one */
 	PKT_secret_key *sk;
 
-	sk = m_alloc_clear( sizeof *sk );
+	sk = xmalloc_clear( sizeof *sk );
 	sk->req_usage = use;
 	if( (rc = get_seckey_byname( sk, NULL, unlock )) ) {
 	  free_secret_key( sk ); sk = NULL;
@@ -130,7 +131,7 @@ build_sk_list( STRLIST locusr, SK_LIST *ret_sk_list,
 	      }
 	    else
 	      {
-		r = m_alloc( sizeof *r );
+		r = xmalloc( sizeof *r );
 		r->sk = sk; sk = NULL;
 		r->next = sk_list;
 		r->mark = 0;
@@ -158,7 +159,7 @@ build_sk_list( STRLIST locusr, SK_LIST *ret_sk_list,
 		log_error(_("skipped \"%s\": duplicated\n"), locusr->d );
                 continue;
 	      }
-	    sk = m_alloc_clear( sizeof *sk );
+	    sk = xmalloc_clear( sizeof *sk );
 	    sk->req_usage = use;
 	    if( (rc = get_seckey_byname( sk, locusr->d, 0 )) )
 	      {
@@ -193,7 +194,7 @@ build_sk_list( STRLIST locusr, SK_LIST *ret_sk_list,
 		    free_secret_key( sk ); sk = NULL;
 		}
 		else {
-		    r = m_alloc( sizeof *r );
+		    r = xmalloc( sizeof *r );
 		    r->sk = sk; sk = NULL;
 		    r->next = sk_list;
 		    r->mark = 0;
