@@ -33,6 +33,7 @@
 #include "filter.h"
 #include "i18n.h"
 #include "options.h"
+#include "status.h"
 
 #ifdef HAVE_DOSISH_SYSTEM
 #define LF "\r\n"
@@ -147,7 +148,7 @@ text_filter( void *opaque, int control,
 	if( tfx->truncated )
 	    log_error(_("can't handle text lines longer than %d characters\n"),
 			MAX_LINELEN );
-	m_free( tfx->buffer );
+	xfree( tfx->buffer );
 	tfx->buffer = NULL;
     }
     else if( control == IOBUFCTRL_DESC )
@@ -177,6 +178,8 @@ copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
     if( !escape_dash )
 	escape_from = 0;
 
+    write_status (STATUS_BEGIN_SIGNING);
+
     for(;;) {
 	maxlen = MAX_LINELEN;
 	n = iobuf_read_line( inp, &buffer, &bufsize, &maxlen );
