@@ -1,7 +1,7 @@
 /* curl-shim.c - Implement a small subset of the curl API in terms of
  * the iobuf HTTP API
  *
- * Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -164,18 +164,23 @@ curl_easy_perform(CURL *curl)
   /* Emulate the libcurl proxy behavior.  If the calling program set a
      proxy, use it.  If it didn't set a proxy or set it to NULL, check
      for one in the environment.  If the calling program explicitly
-     set a null-string proxy, don't set a proxy at all. */
+     set a null-string proxy the http code doesn't use a proxy at
+     all. */
 
   if(curl->proxy)
-    {
-      if(*curl->proxy)
-	proxy=curl->proxy;
-    }
+    proxy=curl->proxy;
   else
     proxy=getenv(HTTP_PROXY_ENV);
 
   if(curl->flags.verbose)
-    fprintf(curl->errors,"* HTTP proxy is \"%s\"\n",proxy?proxy:"null");
+    {
+      fprintf(curl->errors,"* HTTP proxy is \"%s\"\n",proxy?proxy:"null");
+      fprintf(curl->errors,"* HTTP URL is \"%s\"\n",curl->url);
+      fprintf(curl->errors,"* HTTP auth is \"%s\"\n",
+	      curl->auth?curl->auth:"null");
+      fprintf(curl->errors,"* HTTP method is %s\n",
+	      curl->flags.post?"POST":"GET");
+    }
 
   if(curl->flags.post)
     {
@@ -267,7 +272,7 @@ curl_easy_perform(CURL *curl)
                         "0123456789"
 
 char *
-curl_escape(char *str,int length)
+curl_easy_escape(CURL *curl,char *str,int length)
 {
   int len,max,idx,enc_idx=0;
   char *enc;
@@ -306,7 +311,7 @@ curl_escape(char *str,int length)
       else
 	{
 	  char numbuf[5];
-	  sprintf(numbuf,"%%%02X",str[idx]);
+	  sprintf(numbuf,"%%%02X",(unsigned char)str[idx]);
 	  strcpy(&enc[enc_idx],numbuf);
 	  enc_idx+=3;
 	}
