@@ -1,5 +1,5 @@
 /* gpgkeys_finger.c - fetch a key via finger
- * Copyright (C) 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -55,15 +56,8 @@
 extern char *optarg;
 extern int optind;
 
-#define GET    0
-#define MAX_LINE 80
-
-static int verbose=0;
-static char path[1024];
-static FILE *input, *output, *console;
-
-#define BEGIN "-----BEGIN PGP PUBLIC KEY BLOCK-----"
-#define END   "-----END PGP PUBLIC KEY BLOCK-----"
+static FILE *input,*output,*console;
+static struct ks_options *opt;
 
 #ifdef _WIN32
 static void
@@ -120,7 +114,7 @@ connect_server (const char *server, unsigned short port)
 
   /* Win32 gethostbyname doesn't handle IP addresses internally, so we
      try inet_addr first on that platform only. */
-  if ((l = inet_addr (server)) != SOCKET_ERROR) 
+  if ((l = inet_addr (server)) != INADDR_NONE) 
     memcpy (&addr.sin_addr, &l, sizeof l);
   else if ((hp = gethostbyname (server))) 
     {
@@ -306,7 +300,7 @@ get_key (char *getkey)
      indicated the requested key anyway. */
   fprintf(output,"KEY 0x%s BEGIN\n",getkey);
 
-  rc = send_request (path, &sock);
+  rc=send_request(opt->opaque,&sock);
   if(rc)
     {
       fprintf(output,"KEY 0x%s FAILED %d\n",getkey, rc);
@@ -330,13 +324,13 @@ get_key (char *getkey)
       
       if(gotit)
         {
-          fputs (line, output);
-          if (!strncmp(line,END,strlen(END)))
+	  print_nocr (output, (const char*)line);
+          if (!strncmp((char*)line,END,strlen(END)))
             break;
         }
-      else if(!strncmp(line,BEGIN,strlen(BEGIN)))
+      else if(!strncmp((char*)line,BEGIN,strlen(BEGIN)))
         {
-          fputs (line,output);
+	  print_nocr(output, (const char*)line);
           gotit=1;
         }
     }
@@ -345,12 +339,12 @@ get_key (char *getkey)
     fprintf (output,"KEY 0x%s END\n", getkey);
   else
     {
-      fprintf (console,"gpgkeys: key %s not found on keyserver\n",getkey);
-      fprintf (output,"KEY 0x%s FAILED %d\n",
-               getkey,KEYSERVER_KEY_NOT_FOUND);
+      fprintf(console,"gpgkeys: no key data found for finger:%s\n",
+	      opt->opaque);
+      fprintf(output,"KEY 0x%s FAILED %d\n",getkey,KEYSERVER_KEY_NOT_FOUND);
     }
 
-  m_free(line);
+  xfree(line);
   iobuf_close (fp_read);
 
   return KEYSERVER_OK;
@@ -368,17 +362,16 @@ show_help (FILE *fp)
 int
 main(int argc,char *argv[])
 {
-  int arg,action=-1,ret=KEYSERVER_INTERNAL_ERROR;
+  int arg,ret=KEYSERVER_INTERNAL_ERROR;
   char line[MAX_LINE];
   char *thekey=NULL;
-  unsigned int timeout=DEFAULT_KEYSERVER_TIMEOUT;
 
   console=stderr;
 
   /* Kludge to implement standard GNU options.  */
   if (argc > 1 && !strcmp (argv[1], "--version"))
     {
-      fputs ("gpgkeys_ldap (GnuPG) " VERSION"\n", stdout);
+      fputs ("gpgkeys_finger (GnuPG) " VERSION"\n", stdout);
       return 0;
     }
   else if (argc > 1 && !strcmp (argv[1], "--help"))
@@ -428,89 +421,38 @@ main(int argc,char *argv[])
   if(output==NULL)
     output=stdout;
 
+  opt=init_ks_options();
+  if(!opt)
+    return KEYSERVER_NO_MEMORY;
+
   /* Get the command and info block */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
     {
-      int version;
-      char commandstr[7];
-      char optionstr[256];
-      char hash;
+      int err;
 
       if(line[0]=='\n')
 	break;
 
-      if(sscanf(line,"%c",&hash)==1 && hash=='#')
-	continue;
-
-      if(sscanf(line,"COMMAND %6s\n",commandstr)==1)
+      err=parse_ks_options(line,opt);
+      if(err>0)
 	{
-	  commandstr[6]='\0';
-
-	  if(strcasecmp(commandstr,"get")==0)
-	    action=GET;
-
-	  continue;
-	}
-
-      if(strncmp(line,"HOST ",5)==0)
-	{
-	  fprintf(console,"gpgkeys: finger://relay/user syntax is not"
-		  " supported.  Use finger:user instead.\n");
-	  ret=KEYSERVER_NOT_SUPPORTED;
+	  ret=err;
 	  goto fail;
 	}
-
-      if(sscanf(line,"OPAQUE %1023s\n",path)==1)
-	{
-	  path[1023]='\0';
-	  continue;
-	}
-
-      if(sscanf(line,"VERSION %d\n",&version)==1)
-	{
-	  if(version!=KEYSERVER_PROTO_VERSION)
-	    {
-	      ret=KEYSERVER_VERSION_ERROR;
-	      goto fail;
-	    }
-
-	  continue;
-	}
-
-      if(sscanf(line,"OPTION %255s\n",optionstr)==1)
-	{
-	  int no=0;
-	  char *start=&optionstr[0];
-
-	  optionstr[255]='\0';
-
-	  if(strncasecmp(optionstr,"no-",3)==0)
-	    {
-	      no=1;
-	      start=&optionstr[3];
-	    }
-
-	  if(strcasecmp(start,"verbose")==0)
-	    {
-	      if(no)
-		verbose--;
-	      else
-		verbose++;
-	    }
-	  else if(strncasecmp(start,"timeout",7)==0)
-	    {
-	      if(no)
-		timeout=0;
-	      else
-		timeout=atoi(&start[8]);
-	    }
-
-	  continue;
-	}
+      else if(err==0)
+	continue;
     }
 
-  if(timeout && register_timeout()==-1)
+  if(opt->host)
+    {
+      fprintf(console,"gpgkeys: finger://relay/user syntax is not"
+	      " supported.  Use finger:user instead.\n");
+      ret=KEYSERVER_NOT_SUPPORTED;
+      goto fail;
+    }
+
+  if(opt->timeout && register_timeout()==-1)
     {
       fprintf(console,"gpgkeys: unable to register timeout handler\n");
       return KEYSERVER_INTERNAL_ERROR;
@@ -519,7 +461,7 @@ main(int argc,char *argv[])
   /* If it's a GET or a SEARCH, the next thing to come in is the
      keyids.  If it's a SEND, then there are no keyids. */
 
-  if(action==GET)
+  if(opt->action==KS_GET)
     {
       /* Eat the rest of the file */
       for(;;)
@@ -555,7 +497,7 @@ main(int argc,char *argv[])
       goto fail;
     }
 
-  if(!thekey || !*path)
+  if(!thekey || !opt->opaque)
     {
       fprintf(console,"gpgkeys: invalid keyserver instructions\n");
       goto fail;
@@ -566,16 +508,15 @@ main(int argc,char *argv[])
   fprintf(output,"VERSION %d\n",KEYSERVER_PROTO_VERSION);
   fprintf(output,"PROGRAM %s\n\n",VERSION);
 
-  if (verbose>1)
+  if(opt->verbose>1)
     {
-      if(path[0])
-	fprintf(console,"Path:\t\t%s\n",path);
+      fprintf(console,"User:\t\t%s\n",opt->opaque);
       fprintf(console,"Command:\tGET\n");
     }
 
-  set_timeout(timeout);
+  set_timeout(opt->timeout);
 
-  ret = get_key(thekey);
+  ret=get_key(thekey);
 
  fail:
 
@@ -587,5 +528,7 @@ main(int argc,char *argv[])
   if(output!=stdout)
     fclose(output);
 
+  free_ks_options(opt);
+
   return ret;
 }
