@@ -1,5 +1,6 @@
 /* gpgkeys_hkp.c - talk to an HKP keyserver
- * Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -17,6 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * gives permission to link the code of the keyserver helper tools:
+ * gpgkeys_ldap, gpgkeys_curl and gpgkeys_hkp with the OpenSSL
+ * project's "OpenSSL" library (or with modified versions of it that
+ * use the same license as the "OpenSSL" library), and distribute the
+ * linked executables.  You must obey the GNU General Public License
+ * in all respects for all of the code used other than "OpenSSL".  If
+ * you modify this file, you may extend this exception to your version
+ * of the file, but you are not obligated to do so.  If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 
 #include <config.h>
@@ -89,7 +101,17 @@ send_key(int *eof)
   char keyid[17],state[6];
   char line[MAX_LINE];
   char *key=NULL,*encoded_key=NULL;
-  size_t keylen=0,keymax=0;
+  size_t keysize=1;
+
+  key=malloc(1);
+  if(!key)
+    {
+      fprintf(console,"gpgkeys: unable to allocate memory for key\n");
+      ret=KEYSERVER_NO_MEMORY;
+      goto fail;
+    }
+
+  key[0]='\0';
 
   /* Read and throw away input until we see the BEGIN */
 
@@ -121,25 +143,19 @@ send_key(int *eof)
       }
     else
       {
-	if(strlen(line)+keylen>keymax)
+	char *tempkey;
+	keysize+=strlen(line);
+	tempkey=realloc(key,keysize);
+	if(tempkey==NULL)
 	  {
-	    char *tmp;
-
-	    keymax+=200;
-	    tmp=realloc(key,keymax+1);
-	    if(!tmp)
-	      {
-		free(key);
-		fprintf(console,"gpgkeys: out of memory\n");
-		ret=KEYSERVER_NO_MEMORY;
-		goto fail;
-	      }
-
-	    key=tmp;
+	    fprintf(console,"gpgkeys: unable to reallocate for key\n");
+	    ret=KEYSERVER_NO_MEMORY;
+	    goto fail;
 	  }
+	else
+	  key=tempkey;
 
-	strcpy(&key[keylen],line);
-	keylen+=strlen(line);
+	strcat(key,line);
       }
 
   if(!end)
@@ -150,7 +166,7 @@ send_key(int *eof)
       goto fail;
     }
 
-  encoded_key=curl_escape(key,keylen);
+  encoded_key=curl_easy_escape(curl,key,keysize);
   if(!encoded_key)
     {
       fprintf(console,"gpgkeys: out of memory\n");
@@ -301,7 +317,7 @@ get_name(const char *getkey)
 
   memset(&ctx,0,sizeof(ctx));
 
-  searchkey_encoded=curl_escape((char *)getkey,0);
+  searchkey_encoded=curl_easy_escape(curl,(char *)getkey,0);
   if(!searchkey_encoded)
     {
       fprintf(console,"gpgkeys: out of memory\n");
@@ -387,7 +403,7 @@ search_key(const char *searchkey)
     fprintf(console,"gpgkeys: search type is %d, and key is \"%s\"\n",
 	    search_type,searchkey);
 
-  searchkey_encoded=curl_escape((char *)searchkey,0);
+  searchkey_encoded=curl_easy_escape(curl,(char *)searchkey,0);
   if(!searchkey_encoded)
     {
       fprintf(console,"gpgkeys: out of memory\n");
@@ -414,6 +430,12 @@ search_key(const char *searchkey)
     strcat(request,"11371");
   strcat(request,opt->path);
   append_path(request,"/pks/lookup?op=index&options=mr&search=");
+
+  /* HKP keyservers like the 0x to be present when searching by
+     keyid */
+  if(search_type==KS_SEARCH_KEYID_SHORT || search_type==KS_SEARCH_KEYID_LONG)
+    strcat(request,"0x");
+
   strcat(request,searchkey_encoded);
 
   if(search_type!=KS_SEARCH_SUBSTR)
