@@ -21,7 +21,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -66,12 +67,8 @@ static void
 i18n_init(void)
 {
 #ifdef ENABLE_NLS
-#ifdef HAVE_LC_MESSAGES
-  setlocale( LC_MESSAGES, "" );
-#else
   setlocale( LC_ALL, "" );
-#endif
-  bindtextdomain( PACKAGE, G10_LOCALEDIR );
+  bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain( PACKAGE );
 #endif
 }
