@@ -1,5 +1,5 @@
 /* [argparse.c wk 17.06.97] Argument Parser for option handling
- *  Copyright (C) 1998, 1999, 2000, 2001, 2003
+ *  Copyright (C) 1998, 1999, 2000, 2001, 2003,
  *                2004 Free Software Foundation, Inc.
  *
  *  This file is part of GnuPG.
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  *
  *
  * Note: This is an independent version of the one in WkLib
@@ -215,7 +215,7 @@ store_alias( ARGPARSE_ARGS *arg, char *name, char *value )
      * used as lvalue
      */
 #if 0
-    ALIAS_DEF a = m_alloc( sizeof *a );
+    ALIAS_DEF a = xmalloc( sizeof *a );
     a->name = name;
     a->value = value;
     a->next = (ALIAS_DEF)arg->internal.aliases;
@@ -310,7 +310,7 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 			    trim_spaces( p );
 			}
 			if( !p || !*p ) {
-			    m_free( buffer );
+			    xfree( buffer );
 			    arg->r_opt = -10;
 			}
 			else {
@@ -324,7 +324,7 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 		    char *p;
 		    if( !buffer ) {
 			keyword[i] = 0;
-			buffer = m_strdup(keyword);
+			buffer = xstrdup(keyword);
 		    }
 		    else
 			buffer[i] = 0;
@@ -346,7 +346,7 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 			}
 		    }
 		    if( !set_opt_arg(arg, opts[idx].flags, p) )
-			m_free(buffer);
+			xfree(buffer);
 		}
 		break;
 	    }
@@ -401,7 +401,7 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 		    buffer[i++] = c;
 		else {
 		    buflen += 50;
-		    buffer = m_realloc(buffer, buflen);
+		    buffer = xrealloc(buffer, buflen);
 		    buffer[i++] = c;
 		}
 	    }
@@ -409,7 +409,7 @@ optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
 		keyword[i++] = c;
 	    else {
 		buflen = DIM(keyword)+50;
-		buffer = m_alloc(buflen);
+		buffer = xmalloc(buflen);
 		memcpy(buffer, keyword, i);
 		buffer[i++] = c;
 	    }
@@ -914,7 +914,7 @@ default_strusage( int level )
     switch( level ) {
       case 11: p = "foo"; break;
       case 13: p = "0.0"; break;
-      case 14: p = "Copyright (C) 2004 Free Software Foundation, Inc."; break;
+      case 14: p = "Copyright (C) 2006 Free Software Foundation, Inc."; break;
       case 15: p =
 "This program comes with ABSOLUTELY NO WARRANTY.\n"
 "This is free software, and you are welcome to redistribute it\n"
@@ -930,7 +930,7 @@ default_strusage( int level )
 "GNU General Public License for more details.\n\n"
 "You should have received a copy of the GNU General Public License\n"
 "along with this program; if not, write to the Free Software\n"
-"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n";
+"Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.\n";
 	break;
       case 40: /* short and long usage */
       case 41: p = ""; break;
