@@ -1,5 +1,5 @@
 /* fileutil.c -  file utilities
- *	Copyright (C) 1998, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 2003, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,6 +26,10 @@
 #include <string.h>
 #include <assert.h>
 #include <unistd.h>
+#include <sys/types.h>
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
 #include "util.h"
 #include "memory.h"
 #include "ttyio.h"
@@ -50,10 +54,10 @@ make_basename(const char *filepath, const char *inputpath)
 	    if ( !(p=strrchr(filepath, ':')) )
 #endif
 	      {
-		return m_strdup(filepath);
+		return xstrdup(filepath);
 	      }
 
-    return m_strdup(p+1);
+    return xstrdup(p+1);
 }
 
 
@@ -77,18 +81,73 @@ make_dirname(const char *filepath)
 	    if ( !(p=strrchr(filepath, ':')) )
 #endif
 	      {
-		return m_strdup(EXTSEP_S);
+		return xstrdup(EXTSEP_S);
 	      }
 
     dirname_length = p-filepath;
-    dirname = m_alloc(dirname_length+1);
+    dirname = xmalloc(dirname_length+1);
     strncpy(dirname, filepath, dirname_length);
     dirname[dirname_length] = 0;
 
     return dirname;
 }
 
+/* Expand tildes.  Handles both the ~/foo and ~username/foo cases.
+   Returns what the tilde expands to.  *name is advanced to be past
+   the tilde expansion. */
+static char *
+untilde(const char **name)
+{
+  char *home=NULL;
 
+  assert((*name)[0]=='~');
+
+  if((*name)[1]==DIRSEP_C || (*name)[1]=='\0')
+    {
+      /* This is the "~/foo" or "~" case. */
+      char *tmp=getenv("HOME");
+      if(tmp)
+	home=xstrdup(tmp);
+
+#ifdef HAVE_GETPWUID
+      if(!home)
+	{
+	  struct passwd *pwd;
+
+	  pwd=getpwuid(getuid());
+	  if(pwd)
+	    home=xstrdup(pwd->pw_dir);
+	}
+#endif
+      if(home)
+	(*name)++;
+    }
+#ifdef HAVE_GETPWNAM
+  else
+    {
+      /* This is the "~username" case. */
+      char *user,*sep;
+      struct passwd *pwd;
+
+      user=xstrdup((*name)+1);
+
+      sep=strchr(user,DIRSEP_C);
+      if(sep)
+	*sep='\0';
+
+      pwd=getpwnam(user);
+      if(pwd)
+	{
+	  home=xstrdup(pwd->pw_dir);
+	  (*name)+=1+strlen(user);
+	}
+
+      xfree(user);
+    }
+#endif
+
+  return home;
+}
 
 /*
   Construct a filename from the NULL terminated list of parts.  Tilde
@@ -100,7 +159,7 @@ make_filename( const char *first_part, ... )
     va_list arg_ptr ;
     size_t n;
     const char *s;
-    char *name, *home, *p;
+    char *name, *p, *home=NULL;
 
     va_start( arg_ptr, first_part ) ;
     n = strlen(first_part)+1;
@@ -108,25 +167,28 @@ make_filename( const char *first_part, ... )
 	n += strlen(s) + 1;
     va_end(arg_ptr);
 
-    home = NULL;
 #ifndef __riscos__
-    if( *first_part == '~' && first_part[1] == DIRSEP_C
-			   && (home = getenv("HOME")) && *home )
-	n += strlen(home);
+    if(*first_part=='~')
+      {
+	home=untilde(&first_part);
+	if(home)
+	  n+=strlen(home);
+      }
 #endif
-    name = m_alloc(n);
-    p = home ? stpcpy(stpcpy(name,home), first_part+1)
+    name = xmalloc(n);
+    p = home ? stpcpy(stpcpy(name,home), first_part)
 	     : stpcpy(name, first_part);
     va_start( arg_ptr, first_part ) ;
     while( (s=va_arg(arg_ptr, const char *)) )
 	p = stpcpy(stpcpy(p, DIRSEP_S), s);
     va_end(arg_ptr);
+    xfree(home);
 
 #ifndef __riscos__
     return name;
 #else /* __riscos__ */
     p = riscos_gstrans(name);
-    m_free(name);
+    xfree(name);
     return p;
 #endif /* __riscos__ */
 }
@@ -153,8 +215,8 @@ compare_filenames( const char *a, const char *b )
 
     c = ascii_strcasecmp (abuf, bbuf);
 
-    m_free(abuf);
-    m_free(bbuf);
+    xfree(abuf);
+    xfree(bbuf);
 
     return c;
 #endif /* __riscos__ */
@@ -191,6 +253,7 @@ is_file_compressed( const char *s, int *ret_rc )
     IOBUF a;
     byte buf[4];
     int i, rc = 0;
+    int overflow;
 
     struct magic_compress_s {
         size_t len;
@@ -210,7 +273,7 @@ is_file_compressed( const char *s, int *ret_rc )
         return 0;
     }
 
-    if ( iobuf_get_filelength( a ) < 4 ) {
+    if ( iobuf_get_filelength( a, &overflow ) < 4 && !overflow) {
         *ret_rc = 0;
         goto leave;
     }
