@@ -212,8 +212,12 @@ http_wait_response( HTTP_HD hd, unsigned int *ret_status )
     iobuf_ioctl (hd->fp_write, 1, 1, NULL); /* keep the socket open */
     iobuf_close (hd->fp_write);
     hd->fp_write = NULL;
-    if ( !(hd->flags & HTTP_FLAG_NO_SHUTDOWN) )
-        shutdown( hd->sock, 1 );
+    /* We do not want the shutdown code anymore.  It used to be there
+       to support old versions of pksd.  These versions are anyway
+       unusable and the latest releases haven been fixed to properly
+       handle HTTP 1.0. */
+    /* if ( !(hd->flags & HTTP_FLAG_NO_SHUTDOWN) ) */
+    /*     shutdown( hd->sock, 1 ); */
     hd->in_data = 0;
 
     hd->fp_read = iobuf_sockopen( hd->sock , "r" );
@@ -317,8 +321,6 @@ do_parse_uri( PARSED_URI uri, int only_local_part )
 	uri->scheme = p;
 	if(strcmp(uri->scheme,"http")==0)
 	  uri->port = 80;
-	else if(strcmp(uri->scheme,"hkp")==0)
-	  uri->port = 11371;
 	else
 	  return G10ERR_INVALID_URI; /* Unsupported scheme */
 
@@ -574,7 +576,7 @@ send_request( HTTP_HD hd, const char *auth, const char *proxy )
     request=xmalloc(strlen(server)*2 + strlen(p)
 		    + (authstr?strlen(authstr):0)
 		    + (proxy_authstr?strlen(proxy_authstr):0) + 65);
-    if( proxy )
+    if( proxy && *proxy )
       sprintf( request, "%s http://%s:%hu%s%s HTTP/1.0\r\n%s%s",
 	       hd->req_type == HTTP_REQ_GET ? "GET" :
 	       hd->req_type == HTTP_REQ_HEAD? "HEAD":
